" Menu Translations:	Japanese (for Windows)
" Translated By:	MURAOKA Taro <koron.kaoriya@gmail.com>
" Last Change:		15-Jun-2012.
"
" Copyright (C) 2004,12 MURAOKA Taro <koron.kaoriya@gmail.com>
" THIS FILE IS DISTRIBUTED UNDER THE VIM LICENSE.

source <sfile>:p:h/menu_japanese_japan.932.vim
